package dev.fyloz.colorrecipesexplorer.utils

import dev.fyloz.colorrecipesexplorer.SpringUser
import dev.fyloz.colorrecipesexplorer.config.properties.CreSecurityProperties
import io.jsonwebtoken.Jwts
import io.jsonwebtoken.SignatureAlgorithm
import io.jsonwebtoken.io.Encoders
import io.jsonwebtoken.security.Keys
import java.util.*

data class Jwt(
    val subject: String,
    val secret: String,
    val duration: Long,
    val signatureAlgorithm: SignatureAlgorithm = SignatureAlgorithm.HS512
)

fun SpringUser.buildJwt(properties: CreSecurityProperties) =
    Jwt(this.username, properties.jwtSecret, properties.jwtDuration).build()

fun Jwt.build(): String {
    val expirationMs = System.currentTimeMillis() + this.duration
    val expirationDate = Date(expirationMs)

    val base64Secret = Encoders.BASE64.encode(this.secret.toByteArray())
    val key = Keys.hmacShaKeyFor(base64Secret.toByteArray())

    return Jwts.builder()
        .setSubject(this.subject)
        .setExpiration(expirationDate)
        .signWith(key)
        .compact()
}
